/*
 * Copyright (c) 2016-2019,2022-2023 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Tue Aug 23 2016
 */
#include "acmrsim.h"

#include <err.h>

#include "mrsim_c_types.h"
#include "codels.h"


/* --- Task read -------------------------------------------------------- */

/** Codel mrsim_read_init of task read.
 *
 * Triggered by mrsim_start.
 * Yields to mrsim_main.
 */
genom_event
mrsim_read_init(const genom_context self)
{
  return mrsim_main;
}


/** Codel mrsim_read_perm of task read.
 *
 * Triggered by mrsim_main.
 * Yields to mrsim_recv, mrsim_main, mrsim_start.
 */
genom_event
mrsim_read_perm(const mrsim_s *mrsim, const genom_context self)
{
  int s;

  s = mrsim_wait_msg(mrsim, 500/*ms*/);
  if (s < 0) { warn("pty"); return mrsim_main; }

  return s == 0 ? mrsim_main : mrsim_recv;
}


/** Codel mrsim_read_recv of task read.
 *
 * Triggered by mrsim_recv.
 * Yields to mrsim_main, mrsim_recv.
 */
genom_event
mrsim_read_recv(const mrsim_s *mrsim, bool *mrsim_lock,
                const genom_context self)
{
  mrsim_process_msgs(mrsim);
  return mrsim_main;
}


/* --- Activity set_pty ------------------------------------------------- */

/** Codel mrsim_set_pty of activity set_pty.
 *
 * Triggered by mrsim_start.
 * Yields to mrsim_ether.
 * Throws mrsim_e_sys.
 */
genom_event
mrsim_set_pty(const char device[64], mrsim_s **mrsim, bool *mrsim_lock,
              const genom_context self)
{
  if (mrsim_open_pty(*mrsim, device))
    return mrsim_e_sys_error("pty", self);

  warnx("created %s", device);
  return mrsim_ether;
}
